import asyncio
import json
from typing import Callable
from typing import Dict
from typing import Optional
from typing import Union

import pyrogram
from pyrogram import Client
from pyrogram import ContinuePropagation
from pyrogram.raw.base import InputPeer
from pyrogram.raw.functions.channels import GetFullChannel
from pyrogram.raw.functions.messages import GetFullChat
from pyrogram.raw.functions.phone import EditGroupCallParticipant
from pyrogram.raw.functions.phone import GetGroupCall
from pyrogram.raw.functions.phone import GetGroupParticipants
from pyrogram.raw.functions.phone import JoinGroupCall
from pyrogram.raw.functions.phone import LeaveGroupCall
from pyrogram.raw.types import Channel
from pyrogram.raw.types import ChannelForbidden
from pyrogram.raw.types import Chat
from pyrogram.raw.types import ChatForbidden
from pyrogram.raw.types import DataJSON
from pyrogram.raw.types import GroupCall
from pyrogram.raw.types import GroupCallDiscarded
from pyrogram.raw.types import InputChannel
from pyrogram.raw.types import InputGroupCall
from pyrogram.raw.types import InputPeerChannel
from pyrogram.raw.types import MessageActionChatDeleteUser
from pyrogram.raw.types import MessageActionInviteToGroupCall
from pyrogram.raw.types import MessageService
from pyrogram.raw.types import PeerChat
from pyrogram.raw.types import UpdateChannel
from pyrogram.raw.types import UpdateGroupCall
from pyrogram.raw.types import UpdateGroupCallConnection
from pyrogram.raw.types import UpdateGroupCallParticipants
from pyrogram.raw.types import UpdateNewChannelMessage
from pyrogram.raw.types import UpdateNewMessage
from pyrogram.raw.types import Updates

from ..version_manager import VersionManager
from .bridged_client import BridgedClient
from .client_cache import ClientCache


class PyrogramClient(BridgedClient):
    def __init__(
            self,
            cache_duration: int,
            client: Client,
    ):
        self._app: Client = client
        if VersionManager.version_tuple(
                pyrogram.__version__,
        ) > VersionManager.version_tuple(
            '2.0.0',
        ):
            self._app.send = self._app.invoke
        self._cache: ClientCache = ClientCache(
            cache_duration,
            self,
        )

        @self._app.on_raw_update()
        async def on_update(_, update, __, data2):
            if isinstance(
                    update,
                    UpdateGroupCallParticipants,
            ):
                participants = update.participants
                for participant in participants:
                    result = self._cache.set_participants_cache(
                        update.call.id,
                        self.chat_id(participant.peer),
                        participant.muted,
                        participant.volume,
                        participant.can_self_unmute,
                        participant.video is not None or
                        participant.presentation is not None,
                        participant.presentation is not None,
                        participant.video is not None,
                        participant.raise_hand_rating,
                        participant.left,
                    )
                    if result is not None:
                        if 'PARTICIPANTS_HANDLER' in self.HANDLERS_LIST:
                            await self._propagate(
                                'PARTICIPANTS_HANDLER',
                                self._cache.get_chat_id(update.call.id),
                                result,
                                participant.just_joined,
                                participant.left,
                            )
            if isinstance(
                    update,
                    UpdateGroupCall,
            ):
                chat_id = self.chat_id(data2[update.chat_id])
                if isinstance(
                        update.call,
                        GroupCall,
                ):
                    if update.call.schedule_date is None:
                        self._cache.set_cache(
                            chat_id,
                            InputGroupCall(
                                access_hash=update.call.access_hash,
                                id=update.call.id,
                            ),
                        )
                if isinstance(
                        update.call,
                        GroupCallDiscarded,
                ):
                    self._cache.drop_cache(chat_id)
                    if 'CLOSED_HANDLER' in self.HANDLERS_LIST:
                        await self._propagate(
                            'CLOSED_HANDLER',
                            chat_id,
                        )
            if isinstance(
                    update,
                    UpdateChannel,
            ):
                chat_id = self.chat_id(update)
                if len(data2) > 0:
                    if isinstance(
                            data2[update.channel_id],
                            ChannelForbidden,
                    ):
                        self._cache.drop_cache(chat_id)
                        if 'KICK_HANDLER' in self.HANDLERS_LIST:
                            await self._propagate(
                                'KICK_HANDLER',
                                chat_id,
                            )
            if isinstance(
                    update,
                    UpdateNewChannelMessage,
            ) or isinstance(
                update,
                UpdateNewMessage,
            ):
                if isinstance(
                        update.message,
                        MessageService,
                ):
                    if isinstance(
                            update.message.action,
                            MessageActionInviteToGroupCall,
                    ):
                        if 'INVITE_HANDLER' in self.HANDLERS_LIST:
                            await self._propagate(
                                'INVITE_HANDLER',
                                update.message.action,
                            )
                    if isinstance(
                            update.message.action,
                            MessageActionChatDeleteUser,
                    ):
                        if isinstance(
                                update.message.peer_id,
                                PeerChat,
                        ):
                            chat_id = self.chat_id(update.message.peer_id)
                            if isinstance(
                                    data2[update.message.peer_id.chat_id],
                                    ChatForbidden,
                            ):
                                self._cache.drop_cache(chat_id)
                                if 'KICK_HANDLER' in self.HANDLERS_LIST:
                                    await self._propagate(
                                        'KICK_HANDLER',
                                        chat_id,
                                    )
            if isinstance(
                    data2,
                    Dict,
            ):
                for group_id in data2:
                    if isinstance(
                            update,
                            UpdateNewChannelMessage,
                    ) or isinstance(
                        update,
                        UpdateNewMessage,
                    ):
                        if isinstance(
                                update.message,
                                MessageService,
                        ):
                            if isinstance(
                                    data2[group_id],
                                    Channel,
                            ) or isinstance(
                                data2[group_id],
                                Chat,
                            ):
                                chat_id = self.chat_id(data2[group_id])
                                if data2[group_id].left:
                                    self._cache.drop_cache(
                                        chat_id,
                                    )
                                    if 'LEFT_HANDLER' in self.HANDLERS_LIST:
                                        await self._propagate(
                                            'LEFT_HANDLER',
                                            chat_id,
                                        )
            raise ContinuePropagation()

    async def _propagate(self, event_name: str, *args, **kwargs):
        for event in self.HANDLERS_LIST[event_name]:
            asyncio.ensure_future(event(*args, **kwargs))

    def on_closed_voice_chat(self) -> Callable:
        def decorator(func: Callable) -> Callable:
            if self is not None:
                self.HANDLERS_LIST['CLOSED_HANDLER'].append(func)
            return func

        return decorator

    def on_kicked(self) -> Callable:
        def decorator(func: Callable) -> Callable:
            if self is not None:
                self.HANDLERS_LIST['KICK_HANDLER'].append(func)
            return func

        return decorator

    def on_receive_invite(self) -> Callable:
        def decorator(func: Callable) -> Callable:
            if self is not None:
                self.HANDLERS_LIST['INVITE_HANDLER'].append(func)
            return func

        return decorator

    def on_left_group(self) -> Callable:
        def decorator(func: Callable) -> Callable:
            if self is not None:
                self.HANDLERS_LIST['LEFT_HANDLER'].append(func)
            return func

        return decorator

    def on_participants_change(self) -> Callable:
        def decorator(func: Callable) -> Callable:
            if self is not None:
                self.HANDLERS_LIST['PARTICIPANTS_HANDLER'].append(func)
            return func

        return decorator

    async def get_call(
            self,
            chat_id: int,
    ) -> Optional[InputGroupCall]:
        chat = await self._app.resolve_peer(chat_id)
        if isinstance(chat, InputPeerChannel):
            input_call = (
                await self._app.send(
                    GetFullChannel(
                        channel=InputChannel(
                            channel_id=chat.channel_id,
                            access_hash=chat.access_hash,
                        ),
                    ),
                )
            ).full_chat.call
        else:
            input_call = (
                await self._app.send(
                    GetFullChat(chat_id=chat.chat_id),
                )
            ).full_chat.call

        if input_call is not None:
            call: GroupCall = (
                await self._app.send(
                    GetGroupCall(
                        call=input_call,
                        limit=-1,
                    ),
                )
            ).call

            if call.schedule_date is not None:
                return None

        return input_call

    async def get_group_call_participants(
            self,
            chat_id: int,
    ):
        return await self._cache.get_participant_list(
            chat_id,
        )

    async def get_participants(
            self,
            input_call: InputGroupCall,
    ):
        return [
            {
                'user_id': self.chat_id(participant.peer),
                'muted': participant.muted,
                'volume': participant.volume,
                'can_self_unmute': participant.can_self_unmute,
                'video': participant.video,
                'presentation': participant.presentation,
                'raise_hand_rating': participant.raise_hand_rating,
                'left': participant.left,
            } for participant in (
                await self._app.send(
                    GetGroupParticipants(
                        call=input_call,
                        ids=[],
                        sources=[],
                        offset='',
                        limit=500,
                    ),
                )
            ).participants
        ]

    async def join_group_call(
            self,
            chat_id: int,
            json_join: str,
            invite_hash: str,
            have_video: bool,
            join_as: InputPeer,
    ) -> str:
        chat_call = await self._cache.get_full_chat(chat_id)
        if chat_call is not None:
            result: Updates = await self._app.send(
                JoinGroupCall(
                    call=chat_call,
                    params=DataJSON(data=json_join),
                    muted=False,
                    join_as=join_as,
                    video_stopped=have_video,
                    invite_hash=invite_hash,
                ),
            )
            for update in result.updates:
                if isinstance(
                        update,
                        UpdateGroupCallParticipants,
                ):
                    participants = update.participants
                    for participant in participants:
                        self._cache.set_participants_cache(
                            update.call.id,
                            self.chat_id(participant.peer),
                            participant.muted,
                            participant.volume,
                            participant.can_self_unmute,
                            participant.video is not None or
                            participant.presentation is not None,
                            participant.presentation is not None,
                            participant.video is not None,
                            participant.raise_hand_rating,
                            participant.left,
                        )
                if isinstance(update, UpdateGroupCallConnection):
                    return update.params.data

        return json.dumps({'transport': None})

    async def leave_group_call(
            self,
            chat_id: int,
    ):
        chat_call = await self._cache.get_full_chat(chat_id)
        if chat_call is not None:
            await self._app.send(
                LeaveGroupCall(
                    call=chat_call,
                    source=0,
                ),
            )

    async def change_volume(
            self,
            chat_id: int,
            volume: int,
            participant: InputPeer,
    ):
        chat_call = await self._cache.get_full_chat(chat_id)
        if chat_call is not None:
            await self._app.send(
                EditGroupCallParticipant(
                    call=chat_call,
                    participant=participant,
                    muted=False,
                    volume=volume * 100,
                ),
            )

    async def set_call_status(
            self,
            chat_id: int,
            muted_status: Optional[bool],
            paused_status: Optional[bool],
            stopped_status: Optional[bool],
            participant: InputPeer,
    ):
        chat_call = await self._cache.get_full_chat(chat_id)
        if chat_call is not None:
            await self._app.send(
                EditGroupCallParticipant(
                    call=chat_call,
                    participant=participant,
                    muted=muted_status,
                    video_stopped=stopped_status,
                    video_paused=paused_status,
                ),
            )

    async def get_full_chat(self, chat_id: int):
        return await self._cache.get_full_chat(chat_id)

    async def resolve_peer(
            self,
            user_id: Union[int, str],
    ) -> InputPeer:
        return await self._app.resolve_peer(user_id)

    async def get_id(self) -> int:
        return (await self._app.get_me()).id

    def is_connected(self) -> bool:
        return self._app.is_connected

    async def start(self):
        await self._app.start()



from typing import Dict
from typing import Optional

from ntgcalls import InputMode

from ...ffmpeg import build_command
from ...ffmpeg import check_stream
from .smart_stream import SmartStream
from .video_parameters import VideoParameters
from .video_stream import VideoStream


class VideoPiped(SmartStream):
    def __init__(
        self,
        path: str,
        video_parameters: VideoParameters = VideoParameters(),
        headers: Optional[Dict[str, str]] = None,
        additional_ffmpeg_parameters: str = '',
    ):
        self._path = path
        self._video_data = (
            additional_ffmpeg_parameters,
            self._path,
            video_parameters,
            [],
            headers,
        )
        super().__init__(
            stream_video=VideoStream(
                InputMode.Shell,
                ' '.join(
                    build_command(
                        'ffmpeg',
                        *self._video_data,
                    ),
                ),
                video_parameters,
            ),
        )

    async def check_stream(self):
        await check_stream(
            *self._video_data,
        )
        self.stream_video.path = ' '.join(
            build_command(
                'ffmpeg',
                *self._video_data,
            ),
        )



class TooOldPyrogramVersion(Exception):
    def __init__(
            self,
            version_needed: str,
            pyrogram_version: str,
    ):
        super().__init__(
            f'Needed pyrogram {version_needed}+, '
            'actually installed is '
            f'{pyrogram_version}',
        )


class TooOldTelethonVersion(Exception):
    def __init__(
            self,
            version_needed: str,
            telethon_version: str,
    ):
        super().__init__(
            f'Needed telethon {version_needed}+, '
            'actually installed is '
            f'{telethon_version}',
        )


class TooOldHydrogramVersion(Exception):
    def __init__(
            self,
            version_needed: str,
            hydrogram_version: str,
    ):
        super().__init__(
            f'Needed hydrogram {version_needed}+, '
            'actually installed is '
            f'{hydrogram_version}',
        )


class InvalidStreamMode(Exception):
    def __init__(self):
        super().__init__(
            'Invalid stream mode',
        )


class NoMTProtoClientSet(Exception):
    def __init__(self):
        super().__init__(
            'No MTProto client set',
        )


class NoActiveGroupCall(Exception):
    def __init__(self):
        super().__init__(
            'No active group call',
        )


class NotInGroupCallError(Exception):
    def __init__(self):
        super().__init__(
            'The userbot there isn\'t in a group call',
        )


class AlreadyJoinedError(Exception):
    def __init__(self):
        super().__init__(
            'Already joined into group call',
        )


class TelegramServerError(Exception):
    def __init__(self):
        super().__init__(
            'Telegram Server is having some '
            'internal problems',
        )


class ClientNotStarted(Exception):
    def __init__(self):
        super().__init__(
            'Ensure you have started the process with start() '
            'before calling this method',
        )


class PyTgCallsAlreadyRunning(Exception):
    def __init__(self):
        super().__init__(
            'PyTgCalls client is already running',
        )


class TooManyCustomApiDecorators(Exception):
    def __init__(self):
        super().__init__(
            'Too Many Custom Api Decorators',
        )


class GroupCallNotFound(Exception):
    def __init__(
            self,
            chat_id: int,
    ):
        super().__init__(
            f'Group call not found with the chat id {chat_id}',
        )


class InvalidMTProtoClient(Exception):
    def __init__(self):
        super().__init__(
            'Invalid MTProto Client',
        )


class NoVideoSourceFound(Exception):
    def __init__(self, path: str):
        super().__init__(
            f'No video source found on "{path}"',
        )


class InvalidVideoProportion(Exception):
    def __init__(self, message: str):
        super().__init__(
            message,
        )


class NoAudioSourceFound(Exception):
    def __init__(self, path: str):
        super().__init__(
            f'No audio source found on "{path}"',
        )


class UnMuteNeeded(Exception):
    def __init__(self):
        super().__init__(
            'Needed to unmute the userbot',
        )

class GpytranslateException(Exception):
    pass

class TranslationError(GpytranslateException):
    def __init__(self, exception: Exception):
        self.args = (exception,)

    def __str__(self) -> str:
        return f"An error occurred while trying to translate/tts: {self.args[0]!r}"



import re

class VersionManager:
    @staticmethod
    def version_tuple(v):
        list_version = []
        for vmj in v.split('.'):
            list_d = re.findall('[0-9]+', vmj)
            for vmn in list_d:
                list_version.append(int(vmn))
        return tuple(list_version)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2020-2023 (c) Randy W @xtdevs, @xtsea
#
# from : https://github.com/TeamKillerX
# Channel : @RendyProjects
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import openai
import requests
from pymongo import MongoClient
from datetime import datetime as dt

class OpenAiToken:
    def __init__(self, api_key: str = None, mongo_url: str = None):
        self.api_key = api_key
        self.mongo_url = mongo_url
        openai.api_key = self.api_key

    def connect_mongo(self):
        return MongoClient(self.mongo_url)["tiktokbot"]["users"]

    def continue_conversation(
        self,
        owner_author: str="Randy Dev",
        user_id: int = None,
        user_message: str = None
    ):
        collection = self.connect_mongo()
        update_data = {"chat_user_id": user_id}

        collection.update_one({"user_id": user_id}, {"$set": update_data}, upsert=True)
        user_data = collection.find_one({"user_id": user_id})
        owner_base = f"""
        Your name is {owner_author}. A kind and friendly AI assistant that answers in
        a short and concise answer. Give short step-by-step reasoning if required.

        Today is {dt.now():%A %d %B %Y %H:%M}
        """
        if user_data:
            conversation_history = user_data.get("assistant_reply")
            if conversation_history is None:
                conversation = "No conversation history available"
            else:
                conversation = conversation_history
            try:
                response = openai.ChatCompletion.create(
                    messages=[
                        {"role": "assistant", "content": "You are a helpful assistant."},
                        {"role": "user", "content": owner_base},
                        {"role": "assistant", "content": conversation},
                        {"role": "user", "content": user_message},
                    ],
                    model="gpt-3.5-turbo",
                    top_p=0.1,
                    timeout=2.5
                )
                assistant_reply = response["choices"][0]["message"]["content"]
                collection.update_one(
                    {"user_id": user_id},
                    {"$push": {"conversation_history": {"user_message": user_message, "assistant_reply": assistant_reply}}}
                )
            except Exception as e:
                assistant_reply = f"Error processing request: {str(e)}"
        else:
            assistant_reply = "User not found in the database."

        return assistant_reply

    def message_output(self, query: str=None):
        response = openai.Completion.create(
            model="text-davinci-003",
            prompt=f"{query}\n:",
            temperature=0,
            max_tokens=500,
            top_p=1.0,
            frequency_penalty=0.0,
            presence_penalty=0.0,
        )
        return response.choices[0].text

    def chat_message_turbo(
        self,
        query: str=None,
        role: str="user",
        model: str="gpt-3.5-turbo"
    ):
        chat_completion = openai.ChatCompletion.create(
            messages=[{"role": role, "content": query}],
            model=model,
            top_p=0.1,
            timeout=2.5
        )
        return chat_completion

    def photo_output(self, query: str=None):
        response = openai.Image.create(prompt=query, n=1, size="1024x1024")
        return response["data"][0]["url"]

    def client_images_generate(
        self,
        query: str=None,
        model: str="dall-e-3",
        quality: str="standard",
        size: str="1024x1024",
        n: int=1
    ):
        chat_image_generate = openai.Image.create(
            prompt=query,
            model=model,
            quality=quality,
            size=size,
            n=n
        )
        return chat_image_generate

    def audio_transcribe(self, file_path):
        with open(file_path, "rb") as path:
            transcript = openai.Audio.transcribe("whisper-1", path)
        return transcript



import random
from .anime import *
from .waifu import *
from .browsers import Browsers
from .cache import Cache
from .groups import AlreadyJoined
from .groups import ErrorDuringJoin
from .groups import GroupCall
from .groups import GroupCallParticipant
from .groups import JoinedGroupCallParticipant
from .groups import JoinedVoiceChat
from .groups import LeftGroupCallParticipant
from .groups import LeftVoiceChat
from .groups import MutedCall
from .groups import NotInGroupCall
from .groups import UpdatedGroupCallParticipant
from .groups import UpgradeNeeded
from .input_stream import AudioImagePiped
from .input_stream import AudioParameters
from .input_stream import AudioPiped
from .input_stream import AudioQuality
from .input_stream import AudioVideoPiped
from .input_stream import CaptureAudioDevice
from .input_stream import CaptureAVDesktop
from .input_stream import CaptureAVDeviceDesktop
from .input_stream import CaptureVideoDesktop
from .input_stream import InputAudioStream
from .input_stream import InputStream
from .input_stream import InputVideoStream
from .input_stream import VideoParameters
from .input_stream import VideoPiped
from .input_stream import VideoQuality
from .input_stream.quality import HighQualityAudio
from .input_stream.quality import HighQualityVideo
from .input_stream.quality import LowQualityAudio
from .input_stream.quality import LowQualityVideo
from .input_stream.quality import MediumQualityAudio
from .input_stream.quality import MediumQualityVideo
from .stream import ChangedStream
from .stream import MutedStream
from .stream import PausedStream
from .stream import ResumedStream
from .stream import StreamAudioEnded
from .stream import StreamDeleted
from .stream import StreamVideoEnded
from .stream import UnMutedStream
from .update import Update

__all__ = [
    "AlreadyJoined",
    "AudioParameters",
    "AudioImagePiped",
    "AudioPiped",
    "AudioQuality",
    "AudioVideoPiped",
    "Browsers",
    "Cache",
    "ChangedStream",
    "ErrorDuringJoin",
    "GroupCall",
    "GroupCallParticipant",
    "HighQualityAudio",
    "HighQualityVideo",
    "InputAudioStream",
    "InputStream",
    "InputVideoStream",
    "JoinedGroupCallParticipant",
    "JoinedVoiceChat",
    "LowQualityAudio",
    "LowQualityVideo",
    "LeftGroupCallParticipant",
    "LeftVoiceChat",
    "MutedStream",
    "MediumQualityAudio",
    "MediumQualityVideo",
    "NotInGroupCall",
    "PausedStream",
    "ResumedStream",
    "StreamAudioEnded",
    "StreamDeleted",
    "StreamVideoEnded",
    "UnMutedStream",
    "UpdatedGroupCallParticipant",
    "Update",
    "CaptureAudioDevice",
    "CaptureAVDesktop",
    "CaptureAVDeviceDesktop",
    "CaptureVideoDesktop",
    "VideoParameters",
    "VideoPiped",
    "VideoQuality",
    "UpgradeNeeded",
    "MutedCall"
]



import asyncio
import json
from typing import Callable
from typing import Optional
from typing import Union

from telethon import TelegramClient
from telethon.errors import ChannelPrivateError
from telethon.events import Raw
from telethon.tl.functions.channels import GetFullChannelRequest
from telethon.tl.functions.messages import GetFullChatRequest
from telethon.tl.functions.phone import EditGroupCallParticipantRequest
from telethon.tl.functions.phone import GetGroupCallRequest
from telethon.tl.functions.phone import GetGroupParticipantsRequest
from telethon.tl.functions.phone import JoinGroupCallRequest
from telethon.tl.functions.phone import LeaveGroupCallRequest
from telethon.tl.types import ChatForbidden
from telethon.tl.types import DataJSON
from telethon.tl.types import GroupCall
from telethon.tl.types import GroupCallDiscarded
from telethon.tl.types import InputChannel
from telethon.tl.types import InputGroupCall
from telethon.tl.types import InputPeerChannel
from telethon.tl.types import MessageActionChatDeleteUser
from telethon.tl.types import MessageActionInviteToGroupCall
from telethon.tl.types import MessageService
from telethon.tl.types import PeerChat
from telethon.tl.types import TypeInputPeer
from telethon.tl.types import UpdateChannel
from telethon.tl.types import UpdateGroupCall
from telethon.tl.types import UpdateGroupCallConnection
from telethon.tl.types import UpdateGroupCallParticipants
from telethon.tl.types import UpdateNewChannelMessage
from telethon.tl.types import UpdateNewMessage
from telethon.tl.types import Updates

from .bridged_client import BridgedClient
from .client_cache import ClientCache


class TelethonClient(BridgedClient):
    def __init__(
        self,
        cache_duration: int,
        client: TelegramClient,
    ):
        self._app: TelegramClient = client
        self._cache: ClientCache = ClientCache(
            cache_duration,
            self,
        )

        @self._app.on(Raw())
        async def on_update(update):
            if isinstance(
                update,
                UpdateGroupCallParticipants,
            ):
                participants = update.participants
                for participant in participants:
                    result = self._cache.set_participants_cache(
                        update.call.id,
                        self.chat_id(participant.peer),
                        participant.muted,
                        participant.volume,
                        participant.can_self_unmute,
                        participant.video is not None or
                        participant.presentation is not None,
                        participant.presentation is not None,
                        participant.video is not None,
                        participant.raise_hand_rating,
                        participant.left,
                    )
                    if result is not None:
                        if 'PARTICIPANTS_HANDLER' in self.HANDLERS_LIST:
                            await self._propagate(
                                'PARTICIPANTS_HANDLER',
                                self._cache.get_chat_id(update.call.id),
                                result,
                                participant.just_joined,
                                participant.left,
                            )
            if isinstance(
                update,
                UpdateGroupCall,
            ):
                chat_id = self.chat_id(
                    await self._app.get_entity(update.chat_id),
                )
                if isinstance(
                    update.call,
                    GroupCall,
                ):
                    if update.call.schedule_date is None:
                        self._cache.set_cache(
                            chat_id,
                            InputGroupCall(
                                access_hash=update.call.access_hash,
                                id=update.call.id,
                            ),
                        )
                if isinstance(
                    update.call,
                    GroupCallDiscarded,
                ):
                    self._cache.drop_cache(
                        chat_id,
                    )
                    if 'CLOSED_HANDLER' in self.HANDLERS_LIST:
                        await self._propagate(
                            'CLOSED_HANDLER',
                            chat_id,
                        )
            if isinstance(
                update,
                UpdateChannel,
            ):
                chat_id = self.chat_id(update)
                try:
                    await self._app.get_entity(chat_id)
                except ChannelPrivateError:
                    self._cache.drop_cache(chat_id)
                    if 'KICK_HANDLER' in self.HANDLERS_LIST:
                        await self._propagate(
                            'KICK_HANDLER',
                            chat_id,
                        )

            if isinstance(
                update,
                UpdateNewChannelMessage,
            ) or isinstance(
                update,
                UpdateNewMessage,
            ):
                if isinstance(
                    update.message,
                    MessageService,
                ):
                    if isinstance(
                        update.message.action,
                        MessageActionInviteToGroupCall,
                    ):
                        if 'INVITE_HANDLER' in self.HANDLERS_LIST:
                            await self._propagate(
                                'INVITE_HANDLER',
                                update.message.action,
                            )
                    if isinstance(update.message.out, bool):
                        if update.message.out:
                            chat_id = self.chat_id(update.message.peer_id)
                            self._cache.drop_cache(chat_id)
                            if 'LEFT_HANDLER' in self.HANDLERS_LIST:
                                await self._propagate(
                                    'LEFT_HANDLER',
                                    chat_id,
                                )
                    if isinstance(
                        update.message.action,
                        MessageActionChatDeleteUser,
                    ):
                        if isinstance(
                            update.message.peer_id,
                            PeerChat,
                        ):
                            chat_id = self.chat_id(update.message.peer_id)
                            if isinstance(
                                await self._app.get_entity(chat_id),
                                ChatForbidden,
                            ):
                                self._cache.drop_cache(chat_id)
                                if 'KICK_HANDLER' in self.HANDLERS_LIST:
                                    await self._propagate(
                                        'KICK_HANDLER',
                                        chat_id,
                                    )

    async def get_call(
        self,
        chat_id: int,
    ) -> Optional[InputGroupCall]:
        chat = await self._app.get_input_entity(chat_id)
        if isinstance(chat, InputPeerChannel):
            input_call = (
                await self._app(
                    GetFullChannelRequest(
                        InputChannel(
                            chat.channel_id,
                            chat.access_hash,
                        ),
                    ),
                )
            ).full_chat.call
        else:
            input_call = (
                await self._app(
                    GetFullChatRequest(chat_id),
                )
            ).full_chat.call
        if input_call is not None:
            try:
                call: GroupCall = (
                    await self._app(
                        GetGroupCallRequest(
                            call=input_call,
                            limit=-1,
                        ),
                    )
                ).call
                if call.schedule_date is not None:
                    return None
            except Exception as e:
                print(e)
        return input_call

    async def get_group_call_participants(
        self,
        chat_id: int,
    ):
        return await self._cache.get_participant_list(
            chat_id,
        )

    async def get_participants(
        self,
        input_call: InputGroupCall,
    ):
        return [
            {
                'user_id': self.chat_id(participant.peer),
                'muted': participant.muted,
                'volume': participant.volume,
                'can_self_unmute': participant.can_self_unmute,
                'video': participant.video,
                'presentation': participant.presentation,
                'raise_hand_rating': participant.raise_hand_rating,
                'left': participant.left,
            } for participant in (
                await self._app(
                    GetGroupParticipantsRequest(
                        call=input_call,
                        ids=[],
                        sources=[],
                        offset='',
                        limit=500,
                    ),
                )
            ).participants
        ]

    async def join_group_call(
        self,
        chat_id: int,
        json_join: str,
        invite_hash: str,
        have_video: bool,
        join_as: TypeInputPeer,
    ) -> str:
        chat_call = await self._cache.get_full_chat(chat_id)
        if chat_call is not None:
            result: Updates = await self._app(
                JoinGroupCallRequest(
                    call=chat_call,
                    params=DataJSON(data=json_join),
                    muted=False,
                    join_as=join_as,
                    video_stopped=have_video,
                    invite_hash=invite_hash,
                ),
            )
            for update in result.updates:
                if isinstance(
                    update,
                    UpdateGroupCallParticipants,
                ):
                    participants = update.participants
                    for participant in participants:
                        self._cache.set_participants_cache(
                            update.call.id,
                            self.chat_id(participant.peer),
                            participant.muted,
                            participant.volume,
                            participant.can_self_unmute,
                            participant.video is not None or
                            participant.presentation is not None,
                            participant.presentation is not None,
                            participant.video is not None,
                            participant.raise_hand_rating,
                            participant.left,
                        )
                if isinstance(update, UpdateGroupCallConnection):
                    return update.params.data

        return json.dumps({'transport': None})

    async def _propagate(self, event_name: str, *args, **kwargs):
        for event in self.HANDLERS_LIST[event_name]:
            asyncio.ensure_future(event(*args, **kwargs))

    def on_closed_voice_chat(self) -> Callable:
        def decorator(func: Callable) -> Callable:
            if self is not None:
                self.HANDLERS_LIST['CLOSED_HANDLER'].append(func)
            return func
        return decorator

    def on_kicked(self) -> Callable:
        def decorator(func: Callable) -> Callable:
            if self is not None:
                self.HANDLERS_LIST['KICK_HANDLER'].append(func)
            return func
        return decorator

    def on_receive_invite(self) -> Callable:
        def decorator(func: Callable) -> Callable:
            if self is not None:
                self.HANDLERS_LIST['INVITE_HANDLER'].append(func)
            return func
        return decorator

    def on_left_group(self) -> Callable:
        def decorator(func: Callable) -> Callable:
            if self is not None:
                self.HANDLERS_LIST['LEFT_HANDLER'].append(func)
            return func
        return decorator

    def on_participants_change(self) -> Callable:
        def decorator(func: Callable) -> Callable:
            if self is not None:
                self.HANDLERS_LIST['PARTICIPANTS_HANDLER'].append(func)
            return func
        return decorator

    async def leave_group_call(
        self,
        chat_id: int,
    ):
        chat_call = await self._cache.get_full_chat(chat_id)
        if chat_call is not None:
            await self._app(
                LeaveGroupCallRequest(
                    call=chat_call,
                    source=0,
                ),
            )

    async def change_volume(
        self,
        chat_id: int,
        volume: int,
        participant: TypeInputPeer,
    ):
        chat_call = await self._cache.get_full_chat(chat_id)
        if chat_call is not None:
            await self._app(
                EditGroupCallParticipantRequest(
                    call=chat_call,
                    participant=participant,
                    muted=False,
                    volume=volume * 100,
                ),
            )

    async def set_call_status(
        self,
        chat_id: int,
        muted_status: Optional[bool],
        paused_status: Optional[bool],
        stopped_status: Optional[bool],
        participant: TypeInputPeer,
    ):
        chat_call = await self._cache.get_full_chat(chat_id)
        if chat_call is not None:
            await self._app(
                EditGroupCallParticipantRequest(
                    call=chat_call,
                    participant=participant,
                    muted=muted_status,
                    video_stopped=stopped_status,
                    video_paused=paused_status,
                ),
            )

    async def get_full_chat(self, chat_id: int):
        return await self._cache.get_full_chat(chat_id)

    async def resolve_peer(
        self,
        user_id: Union[int, str],
    ) -> TypeInputPeer:
        return await self._app.get_input_entity(user_id)

    async def get_id(self) -> int:
        return (await self._app.get_me()).id

    def is_connected(self) -> bool:
        return self._app.is_connected()

    async def start(self):
        await self._app.start()



import asyncio
import re
import subprocess
from sys import platform

from screeninfo import get_monitors
from screeninfo import ScreenInfoError

from ..types.list import List
from .device_info import DeviceInfo
from .screen_info import ScreenInfo


class MediaDevices:
    @staticmethod
    async def get_screen_devices() -> List:
        list_screens: List = List()
        if platform != 'darwin':
            try:
                for screen in get_monitors():
                    list_screens.append(
                        ScreenInfo(
                            screen.x,
                            screen.y,
                            screen.width,
                            screen.height,
                            screen.is_primary,
                            screen.name,
                        ),
                    )
            except ScreenInfoError:
                pass
        return list_screens

    @staticmethod
    async def get_audio_devices() -> List:
        list_devices: List = List()
        if platform == 'darwin':
            return list_devices
        try:
            if platform == 'win32':
                commands = [
                    'ffmpeg',
                    '-list_devices',
                    'true',
                    '-f',
                    'dshow',
                    '-i',
                    'dummy',
                ]
            else:
                commands = [
                    'pactl',
                    'list',
                    'sources',
                ]
            ffmpeg = await asyncio.create_subprocess_exec(
                *tuple(commands),
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE,
            )
            try:
                stdout, stderr = await asyncio.wait_for(
                    ffmpeg.communicate(),
                    timeout=3,
                )
                result: str = ''
                if platform == 'win32':
                    result = stderr.decode('utf-8')
                elif platform != 'darwin':
                    result = stdout.decode('utf-8')
            except subprocess.TimeoutExpired:
                return list_devices
            if platform == 'win32':
                list_raw = result.split('DirectShow audio devices')[1]
                output = re.findall(
                    '\\[.*?].*?"(.*?)".*?\n\\[.*?].*?"(.*?)"', list_raw,
                )
                for device in output:
                    list_devices.append(
                        DeviceInfo(
                            device[1],
                            device[0],
                        ),
                    )
            else:
                output = re.findall(
                    'Name: (.*?)\n.*?Description: (.*?)\n', result,
                )
                for device in output:
                    list_devices.append(
                        DeviceInfo(
                            device[0],
                            device[1],
                        ),
                    )
        except FileNotFoundError:
            pass
        return list_devices



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2020-2023 (c) Randy W @xtdevs, @xtsea
#
# from : https://github.com/TeamKillerX
# Channel : @RendyProjects
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from typing import Optional, Dict, Any
from pydantic import BaseModel
from . import *
from .pushdb import *
from .reminder import *
from .story import *
from .quote import *
from .tr import *
from .channels import *
from .bot import *
from .decorator import *
from .mental import *
from .__version__ import __version__
from .custom_api import CustomApi
from .py_tgcalls import PyTgCalls
from .stream_type import StreamType
from .sync import idle

from .hackertools.chatgpt import RendyDevChat
from .hackertools.github import GithubUsername
from .hackertools.openai_api import OpenAiToken
from .hackertools.prefixes import CustomPrefixes
from .hackertools.reverse import GoogleReverseImage
from .hackertools.rmbg import RemoveBg
from .hackertools.ocrapi import OcrApiUrl
from .hackertools.ipinfo import WhoisIpHacker

from .extreme.carbon import CarbonSuper
from .extreme.chatbot import ChatbotAi
from .extreme.quotestk import QouteSticker
from .extreme.rayso import CarbonRaySo
from .extreme.userinfo import TelegramUserInfo
from .extreme.webshot import WebShotUrl

from .spamwatch.clients import SibylBan
from .profile.user import ProfileClone
from .api.fullstack import FullStackDev
from .api.private import PrivateApiUrl
from .dl.tiktok import TiktokUrl, faster_tiktok_downloader

class AwesomeCoding(BaseModel):
    gpt3_turbo_url: str = b"\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00c\x00h\x00a\x00t\x00g\x00p\x00t\x003\x00-\x00t\x00u\x00r\x00b\x00o\x00"
    google_ai_url: str = b"\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00g\x00o\x00o\x00g\x00l\x00e\x00-\x00a\x00i\x00"
    opendalle_url: str =  b"\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00o\x00p\x00e\x00n\x00d\x00a\x00l\x00l\x00e\x00"
    anime_styled_url: str =  b"\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00a\x00n\x00i\x00m\x00e\x00-\x00s\x00t\x00y\x00l\x00e\x00d\x00"
    unsplash_url: str =  b"\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00u\x00n\x00s\x00p\x00l\x00a\x00s\x00h\x00"
    chatgpt_model_url: str =  b"\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00c\x00h\x00a\x00t\x00g\x00p\x00t\x00-\x00m\x00o\x00d\x00e\x00l\x00"
    gemini_pro_url: str =  b"\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00g\x00e\x00m\x00i\x00n\x00i\x00-\x00a\x00i\x00-\x00p\x00r\x00o\x00"
    translate_url: str =  b"\xff\xfeh\x00t\x00t\x00p\x00s\x00:\x00/\x00/\x00r\x00a\x00n\x00d\x00y\x00d\x00e\x00v\x00-\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00-\x00a\x00p\x00i\x00.\x00h\x00f\x00.\x00s\x00p\x00a\x00c\x00e\x00/\x00r\x00y\x00u\x00z\x00a\x00k\x00i\x00/\x00t\x00r\x00a\x00n\x00s\x00l\x00a\x00t\x00e\x00"
    default_url: Optional[str] = None
    extra_headers: Optional[Dict[str, Any]] = None
    extra_payload: Optional[Dict[str, Any]] = None

__all__ = [
    "__version__"
    "CustomApi",
    "PyTgCalls",
    "StreamType",
    "RendyDevChat",
    "GithubUsername",
    "OpenAiToken",
    "CustomPrefixes",
    "GoogleReverseImage",
    "RemoveBg",
    "OcrApiUrl",
    "WhoisIpHacker",
    "CarbonSuper",
    "ChatbotAi",
    "QouteSticker",
    "CarbonRaySo",
    "TelegramUserInfo",
    "WebShotUrl",
    "SibylBan",
    "ProfileClone",
    "FullStackDev",
    "PrivateApiUrl",
    "TiktokUrl",
    "faster_tiktok_downloader",
    "BadWordsList",
    "Reminder",
    "_Translator_",
    "QuoteRandom"
]



from .changed_stream import ChangedStream
from .muted_stream import MutedStream
from .paused_stream import PausedStream
from .resumed_stream import ResumedStream
from .stream_audio_ended import StreamAudioEnded
from .stream_deleted import StreamDeleted
from .stream_time import StreamTime
from .stream_video_ended import StreamVideoEnded
from .unmuted_stream import UnMutedStream

__all__ = (
    'ChangedStream',
    'MutedStream',
    'PausedStream',
    'ResumedStream',
    'StreamAudioEnded',
    'StreamDeleted',
    'StreamVideoEnded',
    'UnMutedStream',
    'StreamTime',
)



from typing import Union

from ntgcalls import ConnectionError

from ...exceptions import ClientNotStarted
from ...exceptions import NoMTProtoClientSet
from ...exceptions import NotInGroupCallError
from ...scaffold import Scaffold
from ...to_async import ToAsync
from ...types import ResumedStream


class ResumeStream(Scaffold):
    async def resume_stream(
        self,
        chat_id: Union[int, str],
    ):
        if self._app is None:
            raise NoMTProtoClientSet()

        if not self._is_running:
            raise ClientNotStarted()

        chat_id = await self._resolve_chat_id(chat_id)
        try:
            status = await ToAsync(
                self._binding.resume,
                chat_id,
            )
            await self._on_event_update.propagate(
                'RAW_UPDATE_HANDLER',
                self,
                ResumedStream(chat_id),
            )
            return status
        except ConnectionError:
            raise NotInGroupCallError()



from typing import Callable

from deprecation import deprecated

from ...scaffold import Scaffold


class OnRawUpdate(Scaffold):

    @deprecated(
        deprecated_in='1.0.0.dev1',
        details='This method is no longer supported.',
    )
    def on_raw_update(self) -> Callable:
        pass



import logging
from typing import Optional
from typing import Union

from ntgcalls import ConnectionError
from ntgcalls import FileError
from ntgcalls import InvalidParams

from ...exceptions import AlreadyJoinedError
from ...exceptions import ClientNotStarted
from ...exceptions import NoActiveGroupCall
from ...exceptions import NoMTProtoClientSet
from ...exceptions import TelegramServerError
from ...exceptions import UnMuteNeeded
from ...mtproto import BridgedClient
from ...scaffold import Scaffold
from ...to_async import ToAsync
from ...types import JoinedVoiceChat
from ...types.input_stream.stream import Stream
from ..utilities.stream_params import StreamParams

py_logger = logging.getLogger('ryuzakilib')


class JoinGroupCall(Scaffold):
    async def join_group_call(
        self,
        chat_id: Union[int, str],
        stream: Optional[Stream] = None,
        invite_hash: str = None,
        join_as=None,
    ):
        if join_as is None:
            join_as = self._cache_local_peer

        chat_id = await self._resolve_chat_id(chat_id)
        self._cache_user_peer.put(chat_id, join_as)

        if self._app is None:
            raise NoMTProtoClientSet()

        if not self._is_running:
            raise ClientNotStarted()

        chat_call = await self._app.get_full_chat(
            chat_id,
        )
        if chat_call is None:
            raise NoActiveGroupCall()

        media_description = await StreamParams.get_stream_params(
            stream,
        )

        try:
            call_params: str = await ToAsync(
                self._binding.create_call,
                chat_id,
                media_description,
            )

            result_params = await self._app.join_group_call(
                chat_id,
                call_params,
                invite_hash,
                media_description.video is None,
                self._cache_user_peer.get(chat_id),
            )

            await ToAsync(
                self._binding.connect,
                chat_id,
                result_params,
            )

            participants = await self._app.get_group_call_participants(
                chat_id,
            )

            for x in participants:
                if x.user_id == BridgedClient.chat_id(
                        self._cache_local_peer,
                ):
                    self._need_unmute[chat_id] = x.muted_by_admin

            await self._on_event_update.propagate(
                'RAW_UPDATE_HANDLER',
                self,
                JoinedVoiceChat(chat_id),
            )
        except FileError:
            raise FileNotFoundError()
        except ConnectionError:
            raise AlreadyJoinedError()
        except InvalidParams:
            raise UnMuteNeeded()
        except Exception:
            raise TelegramServerError()



from ntgcalls import InputMode

from ...ffmpeg import build_command
from ...media_devices.screen_info import ScreenInfo
from .smart_stream import SmartStream
from .video_parameters import VideoParameters
from .video_stream import VideoStream


class CaptureVideoDesktop(SmartStream):
    def __init__(
        self,
        screen_info: ScreenInfo,
        video_parameters: VideoParameters = VideoParameters(),
    ):
        self._path = screen_info.build_ffmpeg_command(
            video_parameters.frame_rate,
        )
        super().__init__(
            stream_video=VideoStream(
                InputMode.Shell,
                ' '.join(
                    build_command(
                        'ffmpeg',
                        '',
                        self._path,
                        video_parameters,
                        screen_info.ffmpeg_parameters,
                    ),
                ),
            ),
        )



from deprecation import deprecated

from ..audio_parameters import AudioParameters
from ..audio_quality import AudioQuality


@deprecated(
    deprecated_in='1.0.0.dev6',
    details='This class is no longer supported.'
            'Use pytgcalls.types.AudioParameters.from_quality instead.',
)
class HighQualityAudio(AudioParameters):
    def __init__(self):
        super().__init__(*AudioQuality.HIGH.value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2020-2023 (c) Randy W @xtdevs, @xtsea
#
# from : https://github.com/TeamKillerX
# Channel : @RendyProjects
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import requests
import os
import json
import base64
from pyrogram import Client, filters
from pyrogram.types import Message
from base64 import b64decode as idk

class RendyDevChat:
    def __init__(
        self,
        query: str=None,
        binary="01101000 01110100 01110100 01110000 01110011 00111010 00101111 00101111 01100001 01110000 01101001 00101110 01110011 01100001 01100110 01101111 01101110 01100101 00101110 01100100 01100101 01110110 00101111 01100011 01101000 01100001 01110100 01100111 01110000 01110100",
    ):
        self.query = query
        self.binary = binary

    @staticmethod
    def knowledge_hack(text_code):
        you_dont_know = "".join(text_code)
        number = you_dont_know.split()
        return "".join(chr(int(binary, 2)) for binary in number)

    def get_blacklist_from_file(self):
        try:
            with open("blacklist.json", "r") as file:
                blacklist = json.load(file)
            return set(blacklist)
        except FileNotFoundError:
            return set()

    def get_response(
        self,
        message,
        version: int = 3,
        chat_mode: str = "assistant",
        latest_version: bool = False,
    ):
        if isinstance(message, Message):
            blacklist = self.get_blacklist_from_file()
            if message.from_user.id in blacklist:
                return "Blocked User"
        if latest_version:
            response_url = self.knowledge_hack(self.binary)
            payloads = {
                "message": self.query,
                "version": version,
                "chat_mode": chat_mode,
                "dialog_messages": "[{'bot': '', 'user': ''}]",
            }
            try:
                response = requests.post(
                    f"{response_url}",
                    json=payloads,
                    headers={"Content-Type": "application/json"},
                ).json()
                if not (response and "message" in response):
                    print(response)
                    raise ValueError("Invalid Response from Server")
                return response.get("message")
            except Exception as e:
                return f"Error Api {e}"
        else:
            return f"WTF THIS {self.query}"

    def get_response_model(
        self,
        model_id: int=None,
        is_models: bool=False,
        re_json: bool=False,
        status_ok: bool=False
    ):
        url = "https://randydev-ryuzaki-api.hf.space/ryuzaki/chatgpt-model"
        params = {
            "query": self.query,
            "model_id": model_id,
            "is_models": is_models
        }
        response = requests.post(url, json=params)
        if response.status_code != 200:
            return f"Error status: {response.status_code}"

        if status_ok:
            if re_json:
                check_response = response.json()
            else:
                check_response = response
            return check_response
        else:
            return f"WTF THIS {self.query}"

    def get_response_beta(self, joke: bool = False):
        url = "https://freegptapi.hop.sh/neural/api"
        params = {"query": self.query}
        response = requests.get(url, params=params)
        if response.status_code != 200:
            return f"Error status: {response.status_code}"

        if joke:
            check_response = response.json()
            answer = check_response.get("answer")
            return answer
        else:
            return f"WTF THIS {self.query}"

    def get_response_bing(self, bing: bool = False):
        url = f"https://api.freegpt4.ddns.net/?text={self.query}"
        response = requests.get(url)
        if response.status_code != 200:
            return f"Error status: {response.status_code}"

        if bing:
            check_response = response.text
            return check_response
        else:
            return f"WTF THIS {self.query}"

    def get_response_llama(self, llama: bool = False):
        url = f"https://randydev-ryuzaki-api.hf.space/ryuzaki/llama"
        params = {"query": self.query}
        response = requests.get(url, json=params)
        if response.status_code != 200:
            return f"Error status: {response.status_code}"

        if llama:
            check_response = response.json()
            return check_response["randydev"]["message"]
        else:
            return f"WTF THIS {self.query}"

    def get_response_turbo3(self, turbo3: bool = False):
        url = f"https://randydev-ryuzaki-api.hf.space/ryuzaki/chatgpt3-turbo"
        params = {"query": self.query}
        response = requests.post(url, json=params)
        if response.status_code != 200:
            return f"Error status: {response.status_code}"

        if turbo3:
            check_response = response.json()
            return check_response["randydev"]["message"]
        else:
            return f"WTF THIS {self.query}"

    def _model_list(self, is_list_all=False):
        if is_list_all:
            text = """
```python
.get_response(message, latest_version=True)
.get_response_beta(joke=True)
.get_response_bing(bing=True)
.get_response_model() # parameter model_id: integers and is_models: boolean
.get_response_llama(llama=True)
.get_response_turbo3(turbo3=True)
.get_response_gemini_pro(api_key=api_key, re_json=True, is_gemini_pro=True)
.get_response_google_ai(api_key=api_key, re_json=True, is_chat_bison=True, is_google=True)
.multi_chat_response(api_key=api_key, is_multi_chat=True)
```
"""
            return text
        else:
            return "you can check set is_list_all=True"

    def get_response_gemini_pro(
        self,
        api_key: str=None,
        re_json: bool = False,
        is_gemini_pro: bool = False
    ):
        url = f"https://randydev-ryuzaki-api.hf.space/ryuzaki/gemini-ai-pro"
        headers = {
            "accept": "application/json",
            "api-key": api_key
        }
        params = {"query": self.query}
        response = requests.post(url, headers=headers, json=params)
        if response.status_code != 200:
            return f"Error status: {response.status_code}"

        if is_gemini_pro:
            if re_json:
                check_response = response.json()
            else:
                check_response = response
            return check_response
        else:
            return f"WTF THIS {self.query}"

    def get_response_google_ai(
        self,
        api_key: str=None,
        re_json: bool = False,
        is_chat_bison: bool = False,
        is_google: bool = False
    ):
        if is_chat_bison:
            url = f"https://randydev-ryuzaki-api.hf.space/ryuzaki/v1beta2-google-ai"
        else:
            url = f"https://randydev-ryuzaki-api.hf.space/ryuzaki/google-ai"
        headers = {
            "accept": "application/json",
            "api-key": api_key
        }
        params = {"query": self.query}
        response = requests.post(url, headers=headers, json=params)
        if response.status_code != 200:
            return f"Error status: {response.status_code}"

        if is_google:
            if re_json:
                check_response = response.json()
            else:
                check_response = response
            return check_response
        else:
            return f"WTF THIS {self.query}"

    def get_risma_image_generator(
        self,
        api_key: str=None,
        is_opendalle: bool = False
    ):
        url = f"https://randydev-ryuzaki-api.hf.space/ryuzaki/opendalle"
        headers = {
            "accept": "application/json",
            "api-key": api_key
        }
        payload = {"query": self.query}
        response = requests.post(url, headers=headers, json=payload)
        if response.status_code != 200:
            return f"Error status: {response.status_code}"

        if is_opendalle:
            return response.json()
        else:
            return response

    def multi_chat_response(
        self,
        api_key: str=None,
        is_multi_chat: bool = False
    ):
        if is_multi_chat:
            response_str = self.get_response_gemini_pro(
                api_key=api_key,
                re_json=True,
                is_gemini_pro=True,
            )
            response = response_str["randydev"].get("message")
        elif not is_multi_chat:
            response = self.get_response_llama(llama=True)
        elif not is_multi_chat:
            response = self.get_response_bing(bing=True)
        elif not is_multi_chat:
            response = self.get_response_beta(joke=True)
        elif not is_multi_chat:
            response_str = self.get_response_model(
                model_id=5,
                is_models=True,
                re_json=True,
                status_ok=True
            )
            response = response_str["randydev"].get("message")
        else:
            response_str = self.get_response_google_ai(
                api_key=api_key,
                re_json=True,
                is_chat_bison=True,
                is_google=True
            )
            response = response_str["randydev"].get("message")
        return response



import logging
from math import floor
from time import time
from typing import Any
from typing import List
from typing import Optional

from ..types import Cache
from ..types.groups import GroupCallParticipant
from ..types.participant_list import ParticipantList
from .bridged_client import BridgedClient

py_logger = logging.getLogger('pytgcalls')


class ClientCache:
    def __init__(
        self,
        cache_duration: int,
        app: BridgedClient,
    ):
        self._app: BridgedClient = app
        self._cache_duration = cache_duration
        self._full_chat_cache = Cache()
        self._call_participants_cache = Cache()

    async def get_full_chat(
        self,
        chat_id: int,
    ) -> Optional[Any]:
        full_chat = self._full_chat_cache.get(chat_id)
        if full_chat is not None:
            py_logger.debug('FullChat cache hit for %d', chat_id)
            return full_chat
        else:
            # noinspection PyBroadException
            try:
                py_logger.debug('FullChat cache miss for %d', chat_id)
                full_chat = await self._app.get_call(chat_id)
                self.set_cache(
                    chat_id,
                    full_chat,
                )
                return full_chat
            except Exception:
                pass
        return None

    def set_participants_cache(
        self,
        input_id: int,
        user_id: int,
        muted: Optional[bool],
        volume: Optional[int],
        can_self_unmute: Optional[bool],
        video: Optional[bool],
        screen_sharing: Optional[bool],
        video_camera: Optional[bool],
        raised_hand: Optional[int],
        left: Optional[int],
    ) -> Optional[GroupCallParticipant]:
        chat_id = self.get_chat_id(input_id)
        if chat_id is not None:
            participants: Optional[
                ParticipantList
            ] = self._call_participants_cache.get(
                chat_id,
            )
            if participants is not None:
                if not left:
                    return participants.set_participant(
                        user_id,
                        muted if muted is not None
                        else False,
                        muted != can_self_unmute,
                        video if video is not None
                        else False,
                        screen_sharing if screen_sharing is not None
                        else False,
                        video_camera if video_camera is not None
                        else False,
                        raised_hand is not None,
                        floor(volume / 100) if volume is not None
                        else 100,
                    )
                else:
                    return participants.remove_participant(
                        user_id,
                        muted if muted is not None
                        else False,
                        muted != can_self_unmute,
                        video if video is not None
                        else False,
                        screen_sharing if screen_sharing is not None
                        else False,
                        video_camera if video_camera is not None
                        else False,
                        raised_hand is not None,
                        floor(volume / 100) if volume is not None
                        else 100,
                    )
        return None

    async def get_participant_list(
        self,
        chat_id: int,
    ) -> Optional[List[GroupCallParticipant]]:
        input_call = await self.get_full_chat(
            chat_id,
        )
        if input_call is not None:
            participants: Optional[
                ParticipantList
            ] = self._call_participants_cache.get(
                chat_id,
            )
            if participants is not None:
                last_update = participants.last_mtproto_update
                curr_time = int(time())
                if not (last_update - curr_time > 0):
                    py_logger.debug(
                        'GetParticipant cache miss for %d', chat_id,
                    )
                    try:
                        list_participants = await self._app.get_participants(
                            input_call,
                        )
                        for participant in list_participants:
                            self.set_participants_cache(
                                input_call.id,
                                participant['user_id'],
                                participant['muted'],
                                participant['volume'],
                                participant['can_self_unmute'],
                                participant['video'] is not None or
                                participant['presentation'] is not None,
                                participant['presentation'] is not None,
                                participant['video'] is not None,
                                participant['raise_hand_rating'],
                                participant['left'],
                            )
                        participants.last_mtproto_update = \
                            curr_time + self._cache_duration
                    except Exception as e:
                        py_logger.error('Error for %s in %d', e, chat_id)
                else:
                    py_logger.debug('GetParticipant cache hit for %d', chat_id)
                return participants.get_participants()
        return []

    def get_chat_id(
        self,
        input_group_call_id: int,
    ) -> Optional[int]:
        for key in self._call_participants_cache.keys():
            participants = self._call_participants_cache.get(key)
            if participants is not None:
                if participants.input_id == input_group_call_id:
                    return key
        return None

    def set_cache(
        self,
        chat_id: int,
        input_call: Any,
    ) -> None:
        self._full_chat_cache.put(
            chat_id,
            input_call,
            self._cache_duration,
        )
        self._call_participants_cache.put(
            chat_id,
            ParticipantList(
                input_call.id,
            ),
        )

    def drop_cache(
        self,
        chat_id,
    ) -> None:
        self._full_chat_cache.pop(chat_id)
        self._call_participants_cache.pop(chat_id)



from .user_agent import AgentInfo
from .user_agent import UserAgent


class Browsers:
    def __init__(self):
        # CHROME BASE AGENT
        self._chrome_agent = AgentInfo(
            'Chrome',
            '94.0.4606.71',
        )

        # MOZILLA BASE AGENTS
        self._mozilla_android_agent = AgentInfo(
            'Mozilla',
            '5.0',
            'Linux',
            'Android 10',
        )
        self._mozilla_ios_agent = AgentInfo(
            'Mozilla',
            '5.0',
            'iPhone',
            'CPU iPhone OS 15_0 like Mac OS X',
        )
        self._mozilla_linux_agent = AgentInfo(
            'Mozilla',
            '5.0',
            'X11',
            'Linux x86_64',
        )
        self._mozilla_macos_agent = AgentInfo(
            'Mozilla',
            '5.0',
            'Macintosh',
            'Intel Mac OS X 11_6',
        )
        self._mozilla_windows_agent = AgentInfo(
            'Mozilla',
            '5.0',
            'Windows NT 10.0',
            'Win64',
            'x64',
        )

        # APPLE_WEBKIT BASE AGENT
        self._apple_webkit_agent = AgentInfo(
            'AppleWebKit',
            '537.36',
            'KHTML',
            'like Gecko',
        )
        self._apple_webkit_apple_agent = AgentInfo(
            'AppleWebKit',
            '605.1.15',
            'KHTML',
            'like Gecko',
        )

        # SAFARI BASE AGENT
        self._safari_agent = AgentInfo(
            'Safari',
            '537.36',
        )
        self._safari_mobile_agent = AgentInfo(
            'Mobile Safari',
            '537.36',
        )
        self._safari_macos_agent = AgentInfo(
            'Safari',
            '605.1.15',
        )
        self._safari_ios_agent = AgentInfo(
            'Safari',
            '604.1',
        )

        # EDGE BASE AGENT
        self._edge_android_agent = AgentInfo(
            'EdgA',
            '93.0.961.53',
        )
        self._edge_ios_agent = AgentInfo(
            'EdgiOS',
            '93.961.64',
        )
        self._edge_pc_agent = AgentInfo(
            'Edg',
            '94.0.992.31',
        )
        self._edge_windows_mob_agent = AgentInfo(
            'Edge',
            '40.15254.603',
        )
        self._edge_xbox_agent = AgentInfo(
            'Edge',
            '44.18363.8131',
        )

        # FIREFOX BASE AGENT
        self._firefox_default_agent = AgentInfo(
            'Firefox',
            '92.0',
        )
        self._firefox_ios_agent = AgentInfo(
            'FxiOS',
            '38.0',
        )

        # OPERA BASE AGENT
        self._opera_default_agent = AgentInfo(
            'OPR',
            '79.0.4143.66',
        )
        self._opera_mobile_agent = AgentInfo(
            'OPR',
            '63.3.3216.58675',
        )

    # CHROME
    @property
    def chrome_android(self):
        return str(
            UserAgent([
                self._mozilla_android_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_mobile_agent,
            ]),
        )

    @property
    def chrome_ios(self):
        return str(
            UserAgent([
                self._mozilla_ios_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_agent,
            ]),
        )

    @property
    def chrome_linux(self):
        return str(
            UserAgent([
                self._mozilla_linux_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_agent,
            ]),
        )

    @property
    def chrome_macos(self):
        return str(
            UserAgent([
                self._mozilla_macos_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_agent,
            ]),
        )

    @property
    def chrome_windows(self):
        return str(
            UserAgent([
                self._mozilla_windows_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_agent,
            ]),
        )

    # EDGE
    @property
    def edge_android(self):
        return str(
            UserAgent([
                self._mozilla_android_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_mobile_agent,
                self._edge_android_agent,
            ]),
        )

    @property
    def edge_ios(self):
        return str(
            UserAgent([
                self._mozilla_ios_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_agent,
                self._edge_ios_agent,
            ]),
        )

    @property
    def edge_macos(self):
        return str(
            UserAgent([
                self._mozilla_macos_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_agent,
                self._edge_pc_agent,
            ]),
        )

    @property
    def edge_windows(self):
        return str(
            UserAgent([
                self._mozilla_windows_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_agent,
                self._edge_pc_agent,
            ]),
        )

    @property
    def edge_windows_mobile(self):
        return str(
            UserAgent([
                self._mozilla_windows_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_agent,
                self._edge_windows_mob_agent,
            ]),
        )

    @property
    def edge_xbox_one(self):
        return str(
            UserAgent([
                self._mozilla_windows_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_agent,
                self._edge_xbox_agent,
            ]),
        )

    # FIREFOX
    @property
    def firefox_android(self):
        return str(
            UserAgent([
                self._mozilla_android_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_mobile_agent,
                self._firefox_default_agent,
            ]),
        )

    @property
    def firefox_ios(self):
        return str(
            UserAgent([
                self._mozilla_ios_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_agent,
                self._firefox_ios_agent,
            ]),
        )

    @property
    def firefox_linux(self):
        return str(
            UserAgent([
                self._mozilla_linux_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_agent,
                self._firefox_default_agent,
            ]),
        )

    @property
    def firefox_macos(self):
        return str(
            UserAgent([
                self._mozilla_macos_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_agent,
                self._firefox_default_agent,
            ]),
        )

    @property
    def firefox_windows(self):
        return str(
            UserAgent([
                self._mozilla_windows_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_agent,
                self._firefox_default_agent,
            ]),
        )

    # OPERA
    @property
    def opera_android(self):
        return str(
            UserAgent([
                self._mozilla_android_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_mobile_agent,
                self._opera_mobile_agent,
            ]),
        )

    @property
    def opera_linux(self):
        return str(
            UserAgent([
                self._mozilla_linux_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_agent,
                self._opera_default_agent,
            ]),
        )

    @property
    def opera_macos(self):
        return str(
            UserAgent([
                self._mozilla_macos_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_agent,
                self._opera_default_agent,
            ]),
        )

    @property
    def opera_windows(self):
        return str(
            UserAgent([
                self._mozilla_windows_agent,
                self._apple_webkit_agent,
                self._chrome_agent,
                self._safari_agent,
                self._opera_default_agent,
            ]),
        )

    # SAFARI
    @property
    def safari_ios(self):
        return str(
            UserAgent([
                self._mozilla_ios_agent,
                self._apple_webkit_apple_agent,
                self._chrome_agent,
                self._safari_ios_agent,
            ]),
        )

    @property
    def safari_macos(self):
        return str(
            UserAgent([
                self._mozilla_macos_agent,
                self._apple_webkit_apple_agent,
                self._chrome_agent,
                self._safari_macos_agent,
            ]),
        )



from deprecation import deprecated

from ..video_parameters import VideoParameters
from ..video_quality import VideoQuality


@deprecated(
    deprecated_in='1.0.0.dev6',
    details='This class is no longer supported.'
            'Use pytgcalls.types.VideoParameters.from_quality instead.',
)
class MediumQualityVideo(VideoParameters):
    def __init__(self):
        super().__init__(*VideoQuality.SD_360p.value)



from .audio_image_piped import AudioImagePiped
from .audio_parameters import AudioParameters
from .audio_piped import AudioPiped
from .audio_quality import AudioQuality
from .audio_stream import AudioStream
from .audio_video_piped import AudioVideoPiped
from .capture_audio_device import CaptureAudioDevice
from .capture_av_desktop import CaptureAVDesktop
from .capture_av_device_desktop import CaptureAVDeviceDesktop
from .capture_video_desktop import CaptureVideoDesktop
from .input_audio_stream import InputAudioStream
from .input_stream import InputStream
from .input_video_stream import InputVideoStream
from .smart_stream import SmartStream
from .stream import Stream
from .video_parameters import VideoParameters
from .video_piped import VideoPiped
from .video_quality import VideoQuality
from .video_stream import VideoStream

__all__ = (
    'AudioParameters',
    'AudioImagePiped',
    'AudioPiped',
    'AudioQuality',
    'AudioVideoPiped',
    'InputAudioStream',
    'Stream',
    'InputStream',
    'InputVideoStream',
    'SmartStream',
    'VideoStream',
    'AudioStream',
    'CaptureAudioDevice',
    'CaptureAVDesktop',
    'CaptureAVDeviceDesktop',
    'CaptureVideoDesktop',
    'VideoParameters',
    'VideoPiped',
    'VideoQuality',
)



from deprecation import deprecated

from ...types.update import Update


@deprecated(
    deprecated_in='1.0.0.dev1',
    removed_in='1.0.0.dev1',
    details='This method is no longer supported.',
)
class StreamDeleted(Update):
    def __init__(
            self,
            chat_id: int,
    ):
        super().__init__(chat_id)



import asyncio
from typing import Callable
from typing import Dict
from typing import List


class HandlersHolder:
    def __init__(self):
        self._on_event_update: Dict[str, List[Callable]] = {
            'STREAM_END_HANDLER': [],
            'RAW_UPDATE_HANDLER': [],
            'INVITE_HANDLER': [],
            'KICK_HANDLER': [],
            'CLOSED_HANDLER': [],
            'LEFT_HANDLER': [],
            'PARTICIPANTS_LIST': [],
        }

    async def propagate(
        self,
        event_name: str,
        *args,
        **kwargs,
    ):
        for event in self._on_event_update[event_name]:
            asyncio.ensure_future(event(*args, **kwargs))

    def add_handler(
        self,
        event_name: str,
        func: Callable,
    ):
        self._on_event_update[event_name].append(func)



from .high_quality_audio import HighQualityAudio
from .high_quality_video import HighQualityVideo
from .low_quality_audio import LowQualityAudio
from .low_quality_video import LowQualityVideo
from .medium_quality_audio import MediumQualityAudio
from .medium_quality_video import MediumQualityVideo

__all__ = (
    'HighQualityAudio',
    'HighQualityVideo',
    'LowQualityAudio',
    'LowQualityVideo',
    'MediumQualityAudio',
    'MediumQualityVideo',
)



import logging
from typing import Optional
from typing import Union

from ntgcalls import FileError

from ...exceptions import ClientNotStarted
from ...exceptions import NoMTProtoClientSet
from ...exceptions import NotInGroupCallError
from ...scaffold import Scaffold
from ...to_async import ToAsync
from ...types import ChangedStream
from ...types.input_stream.stream import Stream
from ..utilities.stream_params import StreamParams

py_logger = logging.getLogger('ryuzakilib')


class ChangeStream(Scaffold):
    async def change_stream(
        self,
        chat_id: Union[int, str],
        stream: Optional[Stream] = None,
    ):
        if self._app is None:
            raise NoMTProtoClientSet()

        if not self._is_running:
            raise ClientNotStarted()

        chat_id = await self._resolve_chat_id(chat_id)
        try:
            await ToAsync(
                self._binding.change_stream,
                chat_id,
                await StreamParams.get_stream_params(stream),
            )
        except FileError:
            raise FileNotFoundError()
        except Exception:
            raise NotInGroupCallError()

        await self._on_event_update.propagate(
            'RAW_UPDATE_HANDLER',
            self,
            ChangedStream(chat_id),
        )



from sys import platform


class DeviceInfo:
    def __init__(
        self,
        identifier: str,
        title: str,
    ):
        self.identifier = identifier
        self.title = title
        self.ffmpeg_parameters = ['-f']

    def build_ffmpeg_command(self):
        if platform == 'win32':
            self.ffmpeg_parameters += ['dshow']
            return f'audio={self.identifier}'
        else:
            self.ffmpeg_parameters += ['pulse']
        return self.identifier



from .already_joined import AlreadyJoined
from .error_during_join import ErrorDuringJoin
from .group_call import GroupCall
from .group_call_participant import GroupCallParticipant
from .joined_group_call_participant import JoinedGroupCallParticipant
from .joined_voice_chat import JoinedVoiceChat
from .left_group_call_participant import LeftGroupCallParticipant
from .left_voice_chat import LeftVoiceChat
from .muted_call import MutedCall
from .not_in_group_call import NotInGroupCall
from .updated_group_call_participant import UpdatedGroupCallParticipant
from .upgrade_needed import UpgradeNeeded

__all__ = (
    'AlreadyJoined',
    'ErrorDuringJoin',
    'GroupCall',
    'GroupCallParticipant',
    'JoinedGroupCallParticipant',
    'JoinedVoiceChat',
    'LeftGroupCallParticipant',
    'LeftVoiceChat',
    'NotInGroupCall',
    'UpdatedGroupCallParticipant',
    'UpgradeNeeded',
    'MutedCall',
)



from typing import List
from typing import Optional


class AgentInfo:
    def __init__(
        self,
        name: str,
        version: str,
        device: Optional[str] = None,
        os_name: Optional[str] = None,
        arch_type: Optional[str] = None,
    ):
        self.name: str = name
        self.version: str = version
        self.device: Optional[str] = device
        self.os_name: Optional[str] = os_name
        self.arch_type: Optional[str] = arch_type


class UserAgent:
    def __init__(
        self,
        user_agents: List[AgentInfo],
    ):
        self.user_agents: List[AgentInfo] = user_agents

    def __str__(self):
        return ' '.join([
            f'{user_agent.name}/{user_agent.version}'
            ' (' + '; '.join(
                filter(
                    bool, [
                        user_agent.device,
                        user_agent.os_name,
                        user_agent.arch_type,
                    ],
                ),
            ) + ');'
            for user_agent in self.user_agents
        ]).replace(' ()', '')



import asyncio
import logging
import signal
from signal import SIGABRT
from signal import SIGINT
from signal import signal as signal_fn
from signal import SIGTERM


py_logger = logging.getLogger('ryuzakilib')


is_idling = False

# Signal number to name
signals = {
    k: v for v, k in signal.__dict__.items()
    if v.startswith('SIG') and not v.startswith('SIG_')
}


async def idle():
    global is_idling

    def signal_handler(signum, __):
        global is_idling
        py_logger.info(f'Stop signal received ({signals[signum]}). Exiting...')
        is_idling = False
    for s in (SIGINT, SIGTERM, SIGABRT):
        signal_fn(s, signal_handler)
    is_idling = True
    while is_idling:
        await asyncio.sleep(1)
